#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fileio.h"

int get_record_count(FILE *fp)
{
int marker;
int records;

  marker=ftell(fp);
  fseek(fp,0,SEEK_SET);
  records=read_long(fp);
  fseek(fp,marker,SEEK_SET);

  return records; 
}

int set_record_count(FILE *fp, int c)
{
int marker;

  marker=ftell(fp);
  fseek(fp,0,SEEK_SET);
  write_long(fp,c);
  fseek(fp,marker,SEEK_SET);

  return 0; 
}

int get_record_len(FILE *fp)
{
int marker;
int record_len;

  marker=ftell(fp);
  fseek(fp,4,SEEK_SET);
  record_len=read_long(fp);
  fseek(fp,marker,SEEK_SET);

  return record_len; 
}

int get_file_param(FILE *in,char *name,char *value)
{
int ch,l,f;

  name[0]=0;
  value[0]=0;

  f=0; l=0;
  while(1)
  {
    ch=getc(in);

    if (ch=='\n' || ch=='\r')
    {
      if (f!=0) break;
      continue;
    }
      else
    if (ch==' ' && l==0)
    { continue; }
      else
    if (ch==EOF) break;
      else
    if (f==0 && ch==':')
    {
      name[l]=0;
      f++;
      l=0;
      continue;
    }

    if (f==0)
    { name[l++]=ch; }
      else
    { value[l++]=ch; }
  }

  value[l]=0;
  if (name[0]==0 || value[0]==0) return -1;

  return 0;
}

int write_record(FILE *fp,char *name,char *value)
{
int marker,l,r,t;

  marker=ftell(fp);
  write_long(fp,0);

  write_chars(fp,name);
  putc(0,fp);
  write_chars(fp,value);
  putc(0,fp);

  l=ftell(fp);
  r=l%4;
  if (r!=0)
  {
    for (t=0; t<4-r; t++) putc(0,fp);
    l=ftell(fp);
  }

  fseek(fp,marker,SEEK_SET);
  write_long(fp,l-marker-4);
  fseek(fp,l,SEEK_SET);

  return 0;
}

int add_record(FILE *fp,struct rms_record_t *rms_record)
{
char name[128],value[128];
FILE *in;
int l,marker;

  l=strlen(rms_record->infile);
  while(l>0)
  {
    if (rms_record->infile[l]=='.') break;
    l--;
  }

  l++;

  if (strcasecmp(rms_record->infile+l,"jad")==0)
  {
    strcpy(rms_record->content_type,"application/java-archive");
    rms_record->folder_type=1;
  }
    else
  if (strcasecmp(rms_record->infile+l,"jar")==0)
  {
    strcpy(rms_record->content_type,"application/java-archive");
    rms_record->folder_type=1;
  }
    else
  if (strcasecmp(rms_record->infile+l,"pmd")==0)
  {
    strcpy(rms_record->content_type,"application/x-pmd");
    rms_record->folder_type=0;
  }
    else
  {
    printf("Unknown file extension (%s).  If you'd like this file type added,\n",rms_record->infile+l-1);
    printf("please email mike@mikekohn.net and request it.\n\n");
    return -1;
  }

  if (rms_record->folder_type==1)
  {
    if (rms_record->folder[0]==0) strcpy(rms_record->folder,"MYGAMES");

    in=fopen(rms_record->infile,"rb");
    if (in==0)
    {
      printf("Cannot open input file: %s\n",rms_record->infile);
      return -1;
    }

    if (strcasecmp(rms_record->infile+l,"jad")==0)
    {
      while(get_file_param(in,name,value)==0)
      {
        // printf("name=%s value=%s\n",name,value);
        if (strcasecmp(name,"midlet-version")==0 && rms_record->version[0]==0)
        { strcpy(rms_record->version,value); }
          else
        if (strcasecmp(name,"midlet-name")==0)
        {
          if (rms_record->name[0]==0)
          { strcpy(rms_record->class_name,value); }

          strcpy(rms_record->name,value);
        }
          else
        if (strcasecmp(name,"midlet-jar-size")==0 && rms_record->size==0)
        { rms_record->size=atoi(value); }
          else
        if (strcasecmp(name,"midlet-vendor")==0 && rms_record->vendor[0]==0)
        { strcpy(rms_record->vendor,value); }
          else
        if (strcasecmp(name,"midlet-jar-url")==0 && rms_record->full_path[0]==0)
        {
          sprintf(rms_record->full_path,"%s/%d/%s",rms_record->folder,rms_record->id,value);
        }
      }
    }
      else
    {
      fseek(in,0,SEEK_END);
      rms_record->size=ftell(in);
    }

    fclose(in);

    if (rms_record->name[0]==0) strcpy(rms_record->name,rms_record->infile);
    if (rms_record->version[0]==0) strcpy(rms_record->version,"1.0");
    if (rms_record->vendor[0]==0) strcpy(rms_record->vendor,"Unknown");

    l=read_long(fp);
    fseek(fp,0,SEEK_SET);
    write_long(fp,l+1);
    fseek(fp,0,SEEK_END);
    marker=ftell(fp);

    write_long(fp,0);
    write_record(fp,"Content-Type",rms_record->content_type);
    write_record(fp,"Name",rms_record->name);
    write_record(fp,"Version",rms_record->version);
    write_record(fp,"Vendor",rms_record->vendor);
    sprintf(name,"%d",rms_record->id);
    write_record(fp,"ID",name);
    sprintf(name,"%d",rms_record->folder_type);
    write_record(fp,"Folder-Type",name);
    write_record(fp,"Full-Path",rms_record->full_path);
    write_record(fp,"Class-Name",rms_record->class_name);
    sprintf(name,"%d",rms_record->size);
    write_record(fp,"Size",name);
    write_record(fp,"Jad-URL",rms_record->jad_url);
    write_record(fp,"Delete-URL",rms_record->delete_url);
    write_record(fp,"Preloaded-Content","0");
    l=ftell(fp);
    fseek(fp,marker,SEEK_SET);
    write_long(fp,l-marker-4);
    fseek(fp,l,SEEK_SET);
  }
    else
  if (rms_record->folder_type==0)
  {
    if (rms_record->folder[0]==0) strcpy(rms_record->folder,"MYRINGTONES");

    if (rms_record->name[0]==0) strcpy(rms_record->name,rms_record->infile);
    if (rms_record->version[0]==0) strcpy(rms_record->version,"1.0");
    if (rms_record->vendor[0]==0) strcpy(rms_record->vendor,"Unknown");

    fseek(in,0,SEEK_END);
    rms_record->size=ftell(in);

    l=read_long(fp);
    fseek(fp,0,SEEK_SET);
    write_long(fp,l+1);
    fseek(fp,0,SEEK_END);

    marker=ftell(fp);

    write_long(fp,0);
    write_record(fp,"Content-Type",rms_record->content_type);
    write_record(fp,"Name",rms_record->name);
    write_record(fp,"Version",rms_record->version);
    write_record(fp,"Vendor",rms_record->vendor);
    sprintf(name,"%d",rms_record->id);
    write_record(fp,"ID",name);
    sprintf(name,"%d",rms_record->folder_type);
    write_record(fp,"Folder-Type",name);
    write_record(fp,"Full-Path",rms_record->full_path);
    sprintf(name,"%d",rms_record->size);
    write_record(fp,"Size",name);
    write_record(fp,"Jad-URL",rms_record->jad_url);
    write_record(fp,"Delete-URL",rms_record->delete_url);
    write_record(fp,"Preloaded-Content","0");
    l=ftell(fp);
    fseek(fp,marker,SEEK_SET);
    write_long(fp,l-marker-4);
    fseek(fp,l,SEEK_SET);
  }



  return 0;
}

int list_records(FILE *fp)
{
char column[2048];
int records,record_len;
int column_len;
int t,x,l;

  records=read_long(fp);

  printf("Records: %d\n",records);

  for (t=0; t<records; t++)
  {
    record_len=read_long(fp);
    printf("\n---------------------------\n");
    printf("Record %d | len=%d\n",t,record_len);
    printf("---------------------------\n");

    x=0;
    while (x<record_len)
    {
      column_len=read_long(fp);

      if (column_len==0)
      {
        x=x+record_len;
        continue;
      }

      if ((column_len%4)!=0)
      {
        column_len=column_len+(4-(column_len%4));
      }
      x=x+column_len+4;

// printf("%d/%d\n",column_len,x);

      read_chars(fp,column,column_len);
      for (l=0; l<column_len; l++)
      {
        if (column[l]==0) break;
      }
      l++;
      printf("%s: %s\n",column,column+l);
    }

  }

  return 0;
}

int main(int argc, char *argv[])
{
FILE *fp;
char filename[1024];
struct rms_record_t rms_record;
int t,command;

  printf("\nRMS Util "VERSION"\nCopyright 2004 Michael Kohn (mike@mikekohn.net)\n");
  printf("For more info: http://www.mikekohn.net/\n");
  printf("Warning: Use at your own risk!\n\n");

  command=0;
  filename[0]=0;
  rms_record.folder[0]=0;
  memset(&rms_record,0,sizeof(rms_record));

  for (t=1; t<argc; t++)
  {
    if (strcmp(argv[t],"-list")==0)
    {
      command=1;
    }
      else
    if (strcmp(argv[t],"-add")==0)
    {
      strcpy(rms_record.infile,argv[++t]);
      rms_record.id=atoi(argv[++t]);
      command=2;
    }
      else
    if (strcmp(argv[t],"-vendor")==0)
    {
      strcpy(rms_record.vendor,argv[++t]);
    }
      else
    if (strcmp(argv[t],"-version")==0)
    {
      strcpy(rms_record.version,argv[++t]);
    }
      else
    if (strcmp(argv[t],"-jadurl")==0)
    {
      strcpy(rms_record.jad_url,argv[++t]);
    }
      else
    if (strcmp(argv[t],"-delurl")==0)
    {
      strcpy(rms_record.delete_url,argv[++t]);
    }
      else
    if (strcmp(argv[t],"-folder")==0)
    {
      strcpy(rms_record.folder,argv[++t]);
    }
      else
    if (strcmp(argv[t],"-name")==0)
    {
      strcpy(rms_record.name,argv[++t]);
    }
      else
    if (argv[t][0]!='-')
    {
      strcpy(filename,argv[t]);
    }
      else
    {
      printf("Unknown option: %s\n",argv[t]);
      exit(0);
    }
  }

  if (command==0 || filename[0]==0)
  {
    printf("Usage: rms_util [ -options ] <rms.db file>\n");
    printf("    -list = list records\n");
    printf("    -add <file> <id> = add record\n");
    printf("    -vendor <vendor> = when adding record set Vendor\n");
    printf("    -version <version> = when adding record set Version\n");
    printf("    -jadurl <jad-url> = when adding record set Jad-URL\n");
    printf("    -delurl <del-url> = when adding record set Delete-URL\n");
    printf("    -folder <folder> = when adding record set root folder\n");
    printf("    -name <folder> = when adding record set name\n");
    printf("\n");
    exit(0);
  }

  fp=fopen(filename,"rb+");
  if (fp==0)
  {
    printf("Could not open file for reading: %s\n",filename);
    exit(0);
  }

  if (command==1) list_records(fp);
    else
  if (command==2) add_record(fp,&rms_record);

  fclose(fp);

  return 0;
}

